/*
 * main.c
 */

__asm("	text\n	dc.b	'$Id: main.c,v 1.11 1998/08/23 21:06:18 sharl Exp $'\n	even\n");

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/socket.h>
#include <time.h>
#include <err.h>

#include "getopt.h"
#include "config.h"
#include "util.h"
#include "socket.h"
#include "console.h"
#include "ircdef.h"
#include "channel.h"

char *__progname;
#ifdef DEBUG
int debug = 0;
#endif

static struct option long_opt[] = {
#ifdef DEBUG
	{"debug", no_argument, NULL, 'd'},
#endif
	{"help", no_argument, NULL, 'h'},
	{"nick", required_argument, NULL, 'n'},
	{"port", required_argument, NULL, 'p'},
	{"supprt", no_argument, NULL, 's'},
	{"timeout", required_argument, NULL, 't'},
	{"version", no_argument, NULL, 'v'},
	{NULL}
};

/****************************************
	gp@
****************************************/
static volatile void
usage()
{
	fprintf(stderr, "usage: " PROGNAME " [options] [server name] [port]\n"
	       "options:\n"
#ifdef DEBUG
	       "-d, --debug		debug mode\n"
#endif
	       "-h, --help		show this\n"
	       "-n, --nick nick		set nick\n"
	       "-p, --port port		set port number\n"
	       "-s, --support		show keyword ." PROGNAME "rc supported\n"
	       "-t, --timeout sec	set active timeout seconds\n"
	       "-v, --version		show version\n");
	exit(EXIT_FAILURE);
}

/****************************************
	C
****************************************/
int
main(int argc, char *argv[])
{
	int c;
	int sock;

	__progname = argv[0];

	if (ReadConfig() < 0)
		return EXIT_FAILURE;

	while ((c = getopt_long(argc, argv, "dhn:p:st:v", long_opt, 0)) != EOF) {
		switch (c) {
			case 'd':
#ifdef DEBUG
				debug++;
#endif
				break;
			case 'h':
				usage();
			case 'n':
				irc_nick = xrealloc(irc_nick, strlen(optarg) + 1);
				strcpy(irc_nick, optarg);
				break;
			case 'p':
				irc_port = atol(optarg);
				break;
			case 's':
				{
					DIREC *p;

					for (p = directive; p->name; p++)
						printf("%s\n", p->name);
				}
				return EXIT_SUCCESS;
			case 't':
				irc_timeout = atol(optarg);
				break;
			case 'v':
				fprintf(stderr, PROGNAME " " VERSION " (C)1998 Sharl <sharl@www.ufo.co.jp>\n");
				exit(EXIT_FAILURE);
			default:
				usage();
		}
	}
	argc -= optind;
	argv += optind;

	/* set irc_server from command line */
	if (argc > 0) {
		irc_server = xrealloc(irc_server, strlen(argv[0]) + 1);
		strcpy(irc_server, argv[0]);
	}
	argc--;
	argv++;
	/* set irc_port from command line */
	if (argc > 0)
		irc_port = atol(argv[0]);

#ifdef DEBUG
	if (debug) {
		int i;
		char **p;

		printf("server %s\n", irc_server);
		printf("pass   %s\n", irc_pass);
		printf("port   %d\n", irc_port);
		printf("nick   %s\n", irc_nick);
		printf("user   %s\n", irc_user);
		printf("name   %s\n", irc_name);
		printf("info   %s\n", irc_info);
		printf("quit   %s\n", irc_quit_message);
		printf("key    %s\n", irc_keyword);
		printf("bell   %s\n", irc_keyword_bell);
		printf("bold   %s\n", irc_keyword_bold);
		for (i = 0; i < IRC_JOIN_MAX && irc_channel[i]; i++)
			printf("chan   %s\n", irc_channel[i]);
		if (irc_autonicks)
			for (p = irc_autonick; *p; p++)
				printf("anick  %s\n", *p);
		if (irc_autochanops)
			for (p = irc_autochanop; *p; p++)
				printf("achop  %s\n", *p);
		if (irc_isons)
			for (p = irc_ison; *p; p++)
				printf("ison   %s\n", *p);
		printf("ison/s %d\n", irc_ison_interval);
	}
#endif

	{
		int err = 0;

		if (!irc_server)
			fprintf(stderr, RC_SERVER " is NOT set.\n"), err++;
		if (!irc_nick)
			fprintf(stderr, RC_NICK " is NOT set.\n"), err++;
		if (!irc_user)
			fprintf(stderr, RC_USER " is NOT set.\n"), err++;
		if (!irc_name)
			fprintf(stderr, RC_NAME " is NOT set.\n"), err++;
		if (!irc_info)
			fprintf(stderr, RC_INFO " is NOT set.\n"), err++;
		if (err)
			return EXIT_FAILURE;
	}
	if (!irc_quit_message)
		irc_quit_message = "I Quit";

	if ((sock = irc_connect()) < 0)
		return EXIT_FAILURE;
	init_console();
	if (irc_identify(sock)) {
		time_t t_ison;

		irc_initial_join(sock);
		t_ison = timeout = time(NULL);
		while (1) {
			int len;
			time_t tim;

			tim = time(NULL);
			if (tim - timeout > irc_timeout) {
				printf("\033[32mActive timeout occurred.\033[m\n");
				irc_quit(sock, "Active timeout");
				break;
			}
			if ((len = socklen(sock, 0)) > 0)
				if (irc_read(sock) == FALSE)
					break;
			if (irc_console(sock) == FALSE)
				break;
			if (irc_isons != NULL && ison == FALSE
				&& t_ison != tim && tim % irc_ison_interval == 0) {
				t_ison = tim;
				ison = TRUE;
#ifdef DEBUG
				if (debug)
					printf("%s", irc_ison_command);
#endif
				write(sock, irc_ison_command, strlen(irc_ison_command));
			}
		}
	}
	irc_disconnect(sock);

	tini_console();

	return EXIT_SUCCESS;
}
